###############################################################################
##
##      *** COPYRIGHT 2007, CATERPILLAR INC.  ALL RIGHTS RESERVED ***
##
## This program is the property of Caterpillar Inc., includes Caterpillar's
## confidential and trade secret information, and is maintained as an
## unpublished copyrighted work.  It is not to be copied or used by others
## except under license from Caterpillar.  This program is also protected as
## an unpublished work in accordance with the copyright act of 1976.  In the
## event of either inadvertent or deliberate publication, Caterpillar Inc.
## intends to maintain copyright protection for this work under the relevant
## copyright laws pertaining to published works.  The inclusion of a
## copyright notice hereon is precautionary only, and does not imply
## publication or disclosure.
##
###############################################################################

"""
Purpose:
  This script adds a new menu item to Dynasty to call RapidView (a Dynamics 3D
  Visualization program)

Prerequisites:
  
"""

###############################################################################
##
## Import necessary modules.
##
###############################################################################
import DynPy
import os, subprocess, sys

file_ext = ".rpv"
program_name = "RapidView.exe"
RapidView_folder = "E:\\RapidView_portable"

class RapidViewPlugin(DynPy.Callback):
    title = "RapidView"

    def __init__(self):
        DynPy.Callback.__init__(self, self.title)

    def generate_new_file_name(self):
        import ntpath, os.path
        model_path = DynPy.Model.GetActiveModel().GetModelPath()
        folder_path, file_path = ntpath.split(model_path)
        model_name = os.path.splitext(file_path)[0]

        # find the first non-duplicate name
        for i in range(1000):
            fname = ""
            if i == 0:
                fname = model_name + file_ext
            else:
                postfix = '{:03d}'.format(i)
                fname = model_name + "__" + postfix + file_ext

            if not os.path.isfile(fname):
                return folder_path + "\\" + fname
        
        # remove all generated .rpv files
        for i in range(1000):
            if i == 0:
                fname = model_name + file_ext
            else:
                postfix = '{:03d}'.format(i)
                fname = model_name + "__" + postfix + file_ext

            if os.path.isfile(fname):
                os.remove(fname)

        return folder_path + "\\" + model_name + file_ext

    def setupInfo(self, fileName):
        model = DynPy.Model.GetActiveModel()
        all_comp = model.GetAllComponents()

        fo = open(fileName, 'w')
        output_buf = ""

        # get time points
        timePoints = DynPy.Results.Time().GetXValues().GetFloatVectorValue()
        step_num = len(timePoints)

        # count number of rigid, flexible bodies + animation ground
        comp_num = sum(1 for comp in all_comp if (comp.GetType() == 'RBODY3D' or comp.GetType() == 'FBODY3D' or comp.GetType() == 'ANIGRND'))

        output_buf += str(step_num) + " " + str(comp_num) + "\n"
        for val in timePoints:
            output_buf += str(val) + '\n'
        #output_buf += "\n"

        for comp in all_comp:
            comp_name = comp.GetName()
            comp_type = comp.GetType()

            if (comp_type != "RBODY3D" and comp_type != "FBODY3D" and comp_type != "ANIGRND"): continue

            output_buf += 'BEGCOMP ' + comp_name + '\n'
            output_buf += comp_type + '\n'

            # get component's file path
            geo_file_path = ""
            if comp_type == 'ANIGRND':
                geo_file_path = comp.data["ANIINFO"].GetColumn("ANIVALUE").GetStringValue()
                output_buf += geo_file_path + '\n'
                output_buf += 'ENDCOMP ' + comp_name + '\n'
                continue
            else:
                # geo_file_path = os.path.abspath(comp.data["ANIINFONEW"].GetColumn("PARTFILE").GetStringValue())
                geo_file_path = comp.data["ANIINFONEW"].GetColumn("PARTFILE").GetStringValue()
                part_scale = comp.data["ANIINFONEW"].GetColumn("PARTSCALE").GetFloatValue()
                part_trans = comp.data["ANIINFONEW"].GetColumn("PARTTRANS").GetStringValue()
                part_rotate = comp.data["ANIINFONEW"].GetColumn("PARTROTATE").GetStringValue()
                
                output_buf += geo_file_path + '\n'
                output_buf += str(part_scale) + '\n'
                output_buf += part_trans + '\n'
                output_buf += part_rotate + '\n'
                
                # if no existing file path
                geo_file_path = geo_file_path.strip(' \t\n\r')
                if geo_file_path == "":
                    output_buf += 'ENDCOMP ' + comp_name + '\n'
                    continue

            # get component's CG
            cgx = comp.data.CGX.GetFloatValue()
            cgy = comp.data.CGY.GetFloatValue()
            cgz = comp.data.CGZ.GetFloatValue()
            output_buf += '{0:.10f} {1:.10f} {2:.10f}'.format(cgx, cgy, cgz) + '\n'

            # get linear displacement at connectors
            # Rigid body at (CGX, CGY, CGZ) and Flexible body at (IX, IY, IZ)
            local_x = None
            local_y = None
            local_z = None
            if comp_type == 'RBODY3D':
                local_x = comp.conn.CGX.LIN_DISP.GetYValues().GetFloatVectorValue()
                local_y = comp.conn.CGY.LIN_DISP.GetYValues().GetFloatVectorValue()
                local_z = comp.conn.CGZ.LIN_DISP.GetYValues().GetFloatVectorValue()
            elif comp_type == 'FBODY3D':
                local_x = comp.conn.IX.LIN_DISP.GetYValues().GetFloatVectorValue()
                local_y = comp.conn.IY.LIN_DISP.GetYValues().GetFloatVectorValue()
                local_z = comp.conn.IZ.LIN_DISP.GetYValues().GetFloatVectorValue()
            
            # get Euler values
            euler0 = comp.int.E0.GetYValues().GetFloatVectorValue()
            euler1 = comp.int.E1.GetYValues().GetFloatVectorValue()
            euler2 = comp.int.E2.GetYValues().GetFloatVectorValue()
            euler3 = comp.int.E3.GetYValues().GetFloatVectorValue()

            for lx, ly, lz, e0, e1, e2, e3 in zip(local_x, local_y, local_z, euler0, euler1, euler2, euler3):
                output_buf += '{0:.10f} {1:.10f} {2:.10f} {3:.10f} {4:.10f} {5:.10f} {6:.10f}'.format(lx, ly, lz, e0, e1, e2, e3) + '\n'

            #output_buf += '\n'
            output_buf += 'ENDCOMP ' + comp_name + '\n'

        fo.write(output_buf)
        fo.close()

    def Run(self):
        "This is the code which will be executed when the menu item is selected."

        # check results existed
        res = DynPy.Results.Time().GetXValues()
        if not res:
            print("Results was not found!")
            return
        
        # write down rapidView's setup infomation to text file
        data_file = self.generate_new_file_name()
        self.setupInfo(data_file)

        # import subprocess, os, sys
        dyn_working_dir = os.getcwd()  
        os.chdir(RapidView_folder)
        #subprocess.call([program_name, data_file])
        subprocess.Popen([program_name, data_file])
        os.chdir(dyn_working_dir)

###############################################################################
#
# This code will be executed when the module is imported.
#
###############################################################################

# Get a pointer to the Dynasty session.
model = DynPy.Model.GetActiveModel()

# Create the menu item callback.
menu_item = RapidViewPlugin()

# Attach the menu item to Dynasty.
model.AddMenuCallback(menu_item)

